﻿using FMODUnity;
using HarmonyLib;
using Steamworks;
using System;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;
using UnityEngine.SceneManagement;
using System;
using System.Globalization;
using System.Threading.Tasks;

public class BetterControls : Mod
{
    private const string ModNamePrefix = "<color=#6cb030>Better</color><color=#b03030>Controls</color>";
    private const float DefaultMaxSpeed = 1.5f;
    private const float DefaultWaterDriftSpeed = 1.5f;
    private const string MaxSpeedArgumentIsOutOfRangeMessage = "maxspeed: Requires a factor between 1 and 20. (x times of the default speed)\n" +
                                                               "e.g. \"maxspeed 1.5\" will increase your speed to 150%, while \"maxspeed 20\" will increase your speed to 2000%)";

    private float m_maxSpeedFactor = DefaultMaxSpeed;
    private float m_waterDriftSpeedFactor = DefaultWaterDriftSpeed;

    private bool SailsOpen;
    //private Raft_Network network;
    private static Raft_Network network = ComponentManager<Raft_Network>.Value;
    private bool MotorsOn;
    private bool AnchorsDown;
    static BetterControls instance;

    public void Start()
    {
        instance = this;
        Debug.Log(string.Format("{0} has been loaded!", ModNamePrefix));
    }

    public void OnModUnload()
    {
        var raft = ComponentManager<Raft>.Value;
        if (raft == null)
        {
            return;
        }

        raft.maxSpeed = DefaultMaxSpeed;
        raft.waterDriftSpeed = DefaultWaterDriftSpeed;

        Debug.Log(string.Format("{0} has been unloaded!", ModNamePrefix));
        Destroy(gameObject);
    }

    public void Update()
    {
        if (!LoadSceneManager.IsGameSceneLoaded || !Raft_Network.IsHost)
        {
            return;
        }

        var raft = ComponentManager<Raft>.Value;
        if (raft == null)
        {
            return;
        }

        raft.maxSpeed = DefaultMaxSpeed * m_maxSpeedFactor;
        raft.waterDriftSpeed = DefaultWaterDriftSpeed * m_waterDriftSpeedFactor;
    }

    public void toggleAnchors()
    {
        if (network == null)
            network = ComponentManager<Raft_Network>.Value;
        Anchor_Stationary[] anchors = FindObjectsOfType<Anchor_Stationary>();
        for (int i = 0; i < anchors.Length; i++)
        {
            if (anchors[i].AtBottom != AnchorsDown)
            {

                Message_NetworkBehaviour message = new Message_NetworkBehaviour(Messages.StationaryAnchorUse, anchors[i]);
                if (Raft_Network.IsHost)
                {
                    AccessTools.Method("Anchor_Stationary:Use", null, null).Invoke(anchors[i], null);
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }
        }
        AnchorsDown = !AnchorsDown;
    }

    public void toggleSails()
    {
        if (network == null)
            network = ComponentManager<Raft_Network>.Value;
        List<Sail> allSails = Sail.AllSails;
        for (int i = 0; i < allSails.Count; i++)
        {
            // Close all sails 
            Debug.Log(SailsOpen);
            if (SailsOpen)
            {
                Message_NetworkBehaviour message = new Message_NetworkBehaviour(Messages.Sail_Close, allSails[i]);
                if (Raft_Network.IsHost)
                {
                    AccessTools.Method("Sail:Close", null, null).Invoke(allSails[i], null);
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                // Tell host to close mine
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }
            // Open all sails
            else
            {
                Message_NetworkBehaviour message = new Message_NetworkBehaviour(Messages.Sail_Open, allSails[i]);
                if (Raft_Network.IsHost)
                {
                    allSails[i].Open();
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                // Tell the host to open 
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }

        }
        SailsOpen = !SailsOpen;
    }

    public void toggleEngines()
    {
        if (network == null)
            network = ComponentManager<Raft_Network>.Value;
        // Finding all engines
        MotorWheel[] motors = FindObjectsOfType<MotorWheel>();
        for (int i = 0; i < motors.Length; i++)
        {
            // Checking if they are on already
            if (motors[i].MotorState != MotorsOn)
            {
                // If off then turn them on
                Message_NetworkBehaviour_ID message = new Message_NetworkBehaviour_ID(Messages.MotorWheel_PowerButton, network.NetworkIDManager, motors[i].ObjectIndex);
                if (Raft_Network.IsHost)
                {
                    motors[i].ToggleEngine();
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }

        }
        MotorsOn = !MotorsOn;
    }

    public void toggleEnginesDir()
    {
        if (network == null)
            network = ComponentManager<Raft_Network>.Value;
        foreach (MotorWheel motor in FindObjectsOfType<MotorWheel>())
        {
            Message_NetworkBehaviour_ID message = new Message_NetworkBehaviour_ID(Messages.MotorWheel_Inverse, network.NetworkIDManager, motor.ObjectIndex);
            if (Raft_Network.IsHost)
            {
                motor.InversePushDirection();
                network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                continue;
            }
            network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
        }
    }
    public void SailsRotate(string[] args)
    {
        if (args.Length > 0)
        {
            int value;
            if (int.TryParse(args[0], out value))
                SailsRotate(value);
        }
    }
    public static void SailsRotate(float axis)
    {
        List<Sail> allSails = Sail.AllSails;
        foreach (Sail current in allSails)
        {
            if (Raft_Network.IsHost)
            {
                AccessTools.Method("Sail:Rotate").Invoke(current, new object[] { axis });
            }
            else
            {
                Message message = new Message_Sail_Rotate(Messages.Sail_Rotate, current, axis);
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }
        }
    }
    public static void RotateSailsTo(Sail sail)
    {
        if (network == null)
            network = ComponentManager<Raft_Network>.Value;
        List<Sail> allSails = Sail.AllSails;
        foreach (Sail current in allSails)
        {
            if (sail != current)
            {
                float axis = (sail.LocalRotation - current.LocalRotation);
                if (Raft_Network.IsHost)
                {
                    AccessTools.Method("Sail:Rotate").Invoke(current, new object[] { axis });
                }
                else
                {
                    Message message = new Message_Sail_Rotate(Messages.Sail_Rotate, current, axis);
                    network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                }
            }
        }
    }
    public void printRaftSpeed()
    {
        Raft raft = FindObjectOfType<Raft>();
        object value = Traverse.Create(raft).Field("currentMovementSpeed").GetValue() as object;
        Debug.Log(value.ToString());
    }

    private void setRaftMaxSpeed(string[] args)
    {
        if (args.Length != 1)
        {
            Debug.Log(MaxSpeedArgumentIsOutOfRangeMessage);
            return;
        }

        float maxSpeedFloat = -1f;
        int maxSpeedInt = -1;
        if ((!float.TryParse(args[0], out maxSpeedFloat) && !int.TryParse(args[0], out maxSpeedInt)))
        {
            Debug.Log(MaxSpeedArgumentIsOutOfRangeMessage);
            return;
        }

        if (maxSpeedFloat < 0f)
        {
            maxSpeedFloat = maxSpeedInt;
        }

        if (maxSpeedFloat < 1f || maxSpeedFloat > 20f)
        {
            Debug.Log(MaxSpeedArgumentIsOutOfRangeMessage);
            return;
        }

        m_maxSpeedFactor = maxSpeedFloat;
        m_waterDriftSpeedFactor = maxSpeedFloat;
        Debug.Log(string.Format("{0}: Max speed factor was set to {1}", ModNamePrefix, maxSpeedFloat));
    }

    [ConsoleCommand(name: "toggleAnchors", docs: "Toggle all Anchors ON/OFF at once")]
    public static void toggleAnchorsCommand(string[] args)
    {
        instance.toggleAnchors();
    }

    [ConsoleCommand(name: "toggleSails", docs: "Toggle all Sails ON/OFF at once")]
    public static void toggleSailsCommand(string[] args)
    {
        instance.toggleSails();
    }

    [ConsoleCommand(name: "rotateSails", docs: "Rotate sails by a certain number")]
    public static void rotateSailsCommand(string[] args)
    {
        instance.SailsRotate(args);
    }

    [ConsoleCommand(name: "toggleEngines", docs: "Toggle all Engines ON/OFF at once")]
    public static void toggleEnginesCommand(string[] args)
    {
        instance.toggleEngines();
    }
    [ConsoleCommand(name: "toggleEnginesDir", docs: "Toggle all Engines Direction at once")]
    public static void toggleEnginesDirCommand(string[] args)
    {
        instance.toggleEnginesDir();
    }

    [ConsoleCommand(name: "showSpeed", docs: "Tells you your current speed, 1.5 = default raft speed")]
    public static void showSpeedCommand(string[] args)
    {
        instance.printRaftSpeed();
    }
    [ConsoleCommand(name: "setMaxSpeed", docs: "Set max speed for the raft")]
    public static void backupCommand(string[] args)
    {
        instance.setRaftMaxSpeed(args);
    }
}